# -*- coding: utf-8 -*-
"""
Created on Sun Feb 16 18:01:12 2014

@author: Edgar Haener
http://www.negative-probability.co.uk/runge-kutta-method/
"""

def RK4_1(f, t, y, dt):
    k1=f(t,y)
    k2=(  f( t + dt/2, y + dt *k1/2) )
    k3=(  f( t + dt/2, y + dt *k2/2) )
    k4=(  f( t + dt  , y + dt *k3) )
    fp=dt*(k1 + 2*k2 + 2*k3 + k4)/6
    return fp

#RK4 for two functions of two variables each 
# f(y,z), g(y,z), returning t+dt
def RK4_2(f, g, t, y, z, dt):
    k1=f(t,y,z)
    i1=g(t,y,z)
    k2=(f( t + dt/2, y + dt*k1/2, z + dt*i1/2) )
    i2=(g( t + dt/2, y + dt*k1/2, z + dt*i1/2) )
    k3=(f( t + dt/2, y + dt*k2/2, z + dt*i2/2) )
    i3=(g( t + dt/2, y + dt*k2/2, z + dt*i2/2) )
    k4=(f( t + dt  , y + dt*k3, z + dt*i3) )
    i4=(g( t + dt  , y + dt*k3, z + dt*i3) )
    fp=dt*(k1 + 2*k2 + 2*k3 + k4)/6
    gp=dt*(i1 + 2*i2 + 2*i3 + i4)/6
    return (fp, gp)
    
#RK4 for three functions of three variables each 
# f(x,y,z), g(x,y,z), h(x,y,z), returning value at t+dt
def RK4_3(f, g, h, t, x, y, z, dt):
    k1=f(t,x,y,z)
    i1=g(t,x,y,z)
    l1=h(t,x,y,z)
    k2=( f( t + dt/2, x+dt*k1/2, y + dt*i1/2, z + dt*l1/2) )
    i2=( g( t + dt/2, x+dt*k1/2, y + dt*i1/2, z + dt*l1/2) )
    l2=( h( t + dt/2, x+dt*k1/2, y + dt*i1/2, z + dt*l1/2) )
    k3=( f( t + dt/2, x+dt*k2/2, y + dt*i2/2, z + dt*l2/2) )
    i3=( g( t + dt/2, x+dt*k2/2, y + dt*i2/2, z + dt*l2/2) )
    l3=( h( t + dt/2, x+dt*k2/2, y + dt*i2/2, z + dt*l2/2) )
    k4=( f( t + dt  , x+dt*k3, y + dt*i3, z + dt*l3) )
    i4=( g( t + dt  , x+dt*k3, y + dt*i3, z + dt*l3) )
    l4=( h( t + dt  , x+dt*k3, y + dt*i3, z + dt*l3) )
    fp=dt*(k1 + 2*k2 + 2*k3 + k4)/6
    gp=dt*(i1 + 2*i2 + 2*i3 + i4)/6
    hp=dt*(l1 + 2*l2 + 2*l3 + l4)/6
    return (fp, gp, hp)